{**
	CESYS Software License - Version 1.0 - January 1st, 2016

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The software must only be used to operate hardware manufactured by 
	CESYS GmbH, Herzogenaurach.

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
  
	CeboMsr - Delphi example - Frame I/O.

	Multiple inputs are read multiple times in a single transaction,
	values are printed to the console.
	AO-1 and DP-2 are set multiple times in a single transaction.

	Cebo-Stick example does input transfer only.

	@author Daniel Schroll
}
program FrameIoExample;
	uses
		SysUtils,
		CeboMsr in 'cebomsr\CeboMsr.pas';

	{**
		Do Frame IO (read/write multiple I/O in single calls).
		@param(Device Device to use for test.)
	}
	procedure FrameIoTestLC(Device: TDevice);
		var
			Inputs: TIInputList;
			I: longint;
			InFrame: TInputFrame;
			Outputs: TIOutputList;
			OutFrame: TOutputFrame;
	begin
		// Prepare and fill the list of inputs to read.
		SetLength(Inputs,6);
		Inputs[0] := Device.SingleEndedInputs[0];
		Inputs[1] := Device.SingleEndedInputs[1];
		Inputs[2] := Device.DifferentialInputs[1];
		Inputs[3] := Device.DigitalPorts[0];
		Inputs[4] := Device.DigitalPorts[1];
		Inputs[5] := Device.Counters[0];

		// Setup device with this selection.
		Device.SetupInputFrame(Inputs);

		// Read the values multiple times and write them to the console.
		for I := 0 to 99 do begin

			// Read all inputs into the instance of InputFrame.
			InFrame := Device.ReadFrame();

			// Write results to the console.
			writeln(
				'DigitalPort #0: ' + inttostr(InFrame.GetDigitalPort(0)) + ', ' +
				'DigitalPort #1: ' + inttostr(InFrame.GetDigitalPort(1)) + ', ' +
				'SingleEnded #0: ' + floattostr(InFrame.GetSingleEnded(0)) + ', ' +
				'SingleEnded #1: ' + floattostr(InFrame.GetSingleEnded(1)) + ', ' +
				'Differential #1: ' + floattostr(InFrame.GetDifferential(0)) + ', ' +
				'Counter #0: ' + inttostr(InFrame.GetCounter(0))
			);
		end;
	
		// Write to analog out #1 and digital out #2 in one call.
		// Prepare and set output selection.
		SetLength(Outputs,2);
		Outputs[0] := Device.DigitalPorts[2];
		Outputs[1] := Device.AnalogOutputs[1];

		// Prepare device ...
		Device.SetupOutputFrame(Outputs);

		// Prepare digital port #2 as output.
		Device.DigitalPorts[2].OutputEnableMask := $f;

		// Create instance of OutputFrame. There's no direct construction,
		// as this instance may vary between different device types.
		OutFrame := Device.CreateOutputFrame();

		// Write it to hardware, modify contents in every looping.
		for I := 0 to 99 do begin
			OutFrame.SetAnalogOutput(1, 3.0* sin(2.0*PI*I/100.0));
			OutFrame.SetDigitalPort(2, I);
			Device.WriteFrame(OutFrame);
		end;

		// Reset to power up defaults.
		Device.ResetDevice();
	end;




	{**
		Do frame IO (read/write multiple I/O in single calls).
		@param(Device Device to use for test.)
	}
	procedure FrameIoTestStick(Device: TDevice);
		var
			Inputs: TIInputList;
			I: longint;
			InFrame: TInputFrame;
	begin
		// Prepare and fill the list of inputs to read.
		SetLength(Inputs,4);
		Inputs[0] := Device.SingleEndedInputs[0];
		Inputs[1] := Device.SingleEndedInputs[1];
		Inputs[2] := Device.DigitalPorts[0];
		Inputs[3] := Device.Counters[0];

		// Setup device with this selection.
		Device.SetupInputFrame(Inputs);

		// REad the values multiple times and write them to the console.
		for I := 0 to 99 do begin

			// Read all inputs into the instance of InputFrame.
			InFrame := Device.ReadFrame();

			// Write results to the console.
			writeln(
				'DigitalPort #0: ' + inttostr(InFrame.GetDigitalPort(0)) + ', ' +
				'SingleEnded #0: ' + floattostr(InFrame.GetSingleEnded(0)) + ', ' +
				'SingleEnded #1: ' + floattostr(InFrame.GetSingleEnded(1)) + ', ' +
				'Counter #0: ' + inttostr(InFrame.GetCounter(0))
			);
		end;

		// Reset to power up defaults.
		Device.ResetDevice();
	end;
var
	Devices: TDeviceList;
	Device: TDevice;

begin
	try
		// Search for devices
		Devices := CeboMsr.TLibraryInterface.Enumerate(CeboMsr.DeviceType.AllDevices);

		// If at least one device has been found, use the first one
		if (length(Devices) > 0) then begin
			Device := Devices[0];

			// Open device, nothing can be done without doing this
			Device.Open();

			// *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
			// *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
			// *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
			if (Device.DeviceType = CeboMsr.DeviceType.CeboLC) then begin
				FrameIoTestLC(Device);
			end else if (Device.DeviceType = CeboMsr.DeviceType.CeboStick) then begin
				FrameIoTestStick(Device);
			end else begin
				raise EInvalidOperationException.Create('No supported device detected.');
			end;

			// Finalize device usage, this free's up the device, so it can be used
			// again, including other applications.
			Device.Close();
		end else begin
			writeln('No Devices found!');
		end;
		readln;
	except
		on E: EArgumentException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIndexOutOfRangeException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EInvalidOperationException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIoException do begin
			writeln(E.Message);
			readln;
		end;
	end;
end.
