{**
	CESYS Software License - Version 1.0 - January 1st, 2016

	Permission is hereby granted, free of charge, to any person or organization
	obtaining a copy of the software and accompanying documentation covered by
	this license (the "Software") to use, reproduce, display, distribute,
	execute, and transmit the Software, and to prepare derivative works of the
	Software, and to permit third-parties to whom the Software is furnished to
	do so, all subject to the following:

	The software must only be used to operate hardware manufactured by 
	CESYS GmbH, Herzogenaurach.

	The copyright notices in the Software and this entire statement, including
	the above license grant, this restriction and the following disclaimer,
	must be included in all copies of the Software, in whole or in part, and
	all derivative works of the Software, unless such copies or derivative
	works are solely in the form of machine-executable object code generated by
	a source language processor.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
	SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
	FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
	ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
  
	CeboMsr - Delphi example - Streaming Input.

	Multiple inputs are read multiple times at 300 Hz
	and printed to the console.

	@author Daniel Schroll
}

program StreamingIExample;
	uses
		SysUtils,
		CeboMsr in 'cebomsr\CeboMsr.pas';

	{**
		Read blocks of data from device.
		@param(Device Device to use for test.)
	}
	procedure StreamingInputTestLC(Device: TDevice);
		var
			Inputs: TIInputList;
			I: longint;
			Frames: TInputFrameList;
			InFrame: TInputFrame;
	begin

		// Construct Selection that contains inputs to read from.
		setLength(Inputs,5);
		Inputs[0] := Device.SingleEndedInputs[0];
		Inputs[1] := Device.SingleEndedInputs[1];
		Inputs[2] := Device.DifferentialInputs[1];
		Inputs[3] := Device.DigitalPorts[0];
		Inputs[4] := Device.DigitalPorts[1];

		// Prepare device with this collection
		Device.SetupInputFrame(Inputs);

		// Start sampling
		Device.StartBufferedDataAcquisition(300, 20*25, false);

		// Read 20 x 25 frames using blocked read,
		// this function returns after *all* (25) requested frames are collected.
		for I := 0 to 19 do begin

			// Read 25 frames
			Frames := Device.ReadBlocking(25);

			// Write out the first one
			InFrame := Frames[0];
			writeln(
				'DigitalPort #0: ' + inttostr(InFrame.GetDigitalPort(0)) + ', ' +
				'DigitalPort #1: ' + inttostr(InFrame.GetDigitalPort(1)) + ', ' +
				'SingleEnded #0: ' + floattostr(InFrame.GetSingleEnded(0)) + ' V, ' +
				'SingleEnded #1: ' + floattostr(InFrame.GetSingleEnded(1)) + ' V, ' +
				'Differential #1: ' + floattostr(InFrame.GetDifferential(1)) + ' V'
			);
		end;

		// Stop the DAQ.
		Device.StopDataAcquisition();
	end;

	{**
		Read blocks of data from device.
		@param(Device Device to use for test.)
	}
	procedure StreamingInputTestStick(Device: TDevice);
		var
			Inputs: TIInputList;
			I: longint;
			Frames: TInputFrameList;
			InFrame: TInputFrame;
	begin

		// Construct Selection that contains inputs to read from.
		setLength(Inputs,3);
		Inputs[0] := Device.SingleEndedInputs[0];
		Inputs[1] := Device.SingleEndedInputs[1];
		Inputs[2] := Device.DigitalPorts[0];

		// Prepare device with this collection
		Device.SetupInputFrame(Inputs);

		// Start sampling
		Device.StartBufferedDataAcquisition(300, 20*25, false);

		// Read 20 x 25 frames using blocked read,
		// this function returns after *all* (25) requested frames are collected.
		for I := 0 to 19 do begin

			// Read 25 frames
			Frames := Device.ReadBlocking(25);

			// Write out the first one
			InFrame := Frames[0];
			writeln(
				'DigitalPort #0: ' + inttostr(InFrame.GetDigitalPort(0)) + ', ' +
				'SingleEnded #0: ' + floattostr(InFrame.GetSingleEnded(0)) + ' V, ' +
				'SingleEnded #1: ' + floattostr(InFrame.GetSingleEnded(1)) + ' V'
			);
		end;

		// Stop the DAQ.
		Device.StopDataAcquisition();
	end;

var
	Devices: TDeviceList;
	Device: TDevice;

begin
	try
		// Search for devices
		Devices := CeboMsr.TLibraryInterface.Enumerate(CeboMsr.DeviceType.AllDevices);

		// If at least one device has been found, use the first one
		if (length(Devices) > 0) then begin
			Device := Devices[0];

			// Open device, nothing can be done without doing this
			Device.Open();

			// Do some streaming
			if (Device.DeviceType = CeboMsr.DeviceType.CeboLC) then begin
				StreamingInputTestLC(Device);
			end else if (Device.DeviceType = CeboMsr.DeviceType.CeboStick) then begin
				StreamingInputTestStick(Device);
			end else begin
				raise EInvalidOperationException.Create('No supported device detected.');
			end;

			// Finalize device usage, this free's up the device, so it can be used
			// again, including other applications.
			Device.Close();
		end else begin
			writeln('No Devices found!');
		end;
		readln;
	except
		on E: EArgumentException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIndexOutOfRangeException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EInvalidOperationException do begin
			writeln(E.Message);
			readln;
		end;
		on E: EIoException do begin
			writeln(E.Message);
			readln;
		end;
	end;
end.
